#include "m5272.h"

extern void call(int);
extern void wait(int);
extern void enablecache(void);

void copydata_and_clearbss(void) {
  extern int* _etext;
  extern int* _sdata;
  extern int* _edata;
  extern int* _sbss;
  extern int* _ebss;
  int* src=&_etext;
  int* dst=&_sdata;
  int* bss=&_sbss;
  while(dst<&_edata) {
    *dst=*src;
    dst++;
    src++;
  };
  while(bss<&_ebss) {
    *bss=0;
    bss++;
  };
};

void board_init(void) {
  volatile unsigned char  *mbar;
  mbar = (volatile unsigned char *) MCF_MBAR;

  // stuff that was added for dbug compliance (sim_init)
  mbar_writeShort(MCFSIM_SCR, 0x0003);
  mbar_writeShort(MCFSIM_SPR, 0xFFFF);
  mbar_writeLong(MCFSIM_ICR1, 0x88888888);
  mbar_writeLong(MCFSIM_ICR2, 0x88888888);
  mbar_writeLong(MCFSIM_ICR3, 0x88888888);
  mbar_writeLong(MCFSIM_ICR4, 0x88880000);
  mbar_writeLong(MCFSIM_PITR, 0x00000000);
  mbar_writeByte(MCFSIM_PIVR, 0x00000040);
  
  // more stuff from dbug (gpio_init)
  mbar_writeLong(MCFSIM_PACNT, 0x00000000);
  mbar_writeShort(MCFSIM_PADDR, 0x0000);
  mbar_writeLong(MCFSIM_PBCNT, 0x55554155); // PBCNT is the really important part because it allows the console to work
  mbar_writeShort(MCFSIM_PBDDR, 0x0000);
  mbar_writeLong(MCFSIM_PDCNT, 0x00000000);
  
  // chip selects
  // CS0: FLASH
  mbar_writeLong(MCFSIM_CSBR0, 0xFFE00201);
  //	mbar_writeLong(MCFSIM_CSOR0, 0xFFE0001C);
  mbar_writeLong(MCFSIM_CSOR0, 0xFFE00014);
  
#if 0
  // CS2: External 512K SRAM -- may or may not be there
  mbar_writeLong(MCFSIM_CSBR2, 0x30000001);
  mbar_writeLong(MCFSIM_CSOR2, 0xFFF80000);
#else
  mbar_writeLong(MCFSIM_CSBR2, 0);
  mbar_writeLong(MCFSIM_CSOR2, 0);
#endif
  
  // CS7: SDRAM
  // Match = 0x003xxxxx, SDRAM = 0x400, Cache Line = 0x300, Enable = 1
  mbar_writeLong(MCFSIM_CSBR7, 0x00000701);
  mbar_writeLong(MCFSIM_CSOR7, 0xFFC0007C);

  /* only init SDRAM if not yet active */
  if(!(mbar_readLong(MCFSIM_SDCR) & 2)) {
    /* wait at least 100 microseconds after reset before
       writing to SDRAM resgisters */
    wait(1250); // 1 ms
    mbar_writeLong(MCFSIM_SDTR, 0xF539);
    mbar_writeLong(MCFSIM_SDCR, 0x4211);
    /* perform a dummy write to start controller */
    *((volatile unsigned long *)0) = 0;
    /* wait 'til end of init-sequence */
    while(!(mbar_readLong(MCFSIM_SDCR) & 2));
  };
  
  // CS1, CS2, CS3, CS4, CS5, and CS6 are unused on the M5272C3.
  // CS2 is the only one of these which might ever be used;
  // it would be used for external SRAM, but the M5272C3 has none.
  mbar_writeLong(MCFSIM_CSBR1, 0);
  mbar_writeLong(MCFSIM_CSOR1, 0);
  mbar_writeLong(MCFSIM_CSBR3, 0);
  mbar_writeLong(MCFSIM_CSOR3, 0);
  mbar_writeLong(MCFSIM_CSBR4, 0);
  mbar_writeLong(MCFSIM_CSOR4, 0);
  mbar_writeLong(MCFSIM_CSBR5, 0);
  mbar_writeLong(MCFSIM_CSOR5, 0);
  mbar_writeLong(MCFSIM_CSBR6, 0);
  mbar_writeLong(MCFSIM_CSOR6, 0);

  enablecache();
  copydata_and_clearbss();
};

void recover(void) {};
#include "misc.c"

#define IMAGE_SRC 0xffe06000
#define IMAGE_DST 0x3e0000
#define IMAGE_SIZE 0x100000

void start_main(void) {
  decompress_image((void*)IMAGE_SRC,(void*)IMAGE_DST,IMAGE_SIZE);
  call(IMAGE_DST);
};
